@@ -1,4 +1,5 @@
 use crate::{dom::dom_updater::DomUpdater, Application, Dispatch};
+use std::any::TypeId;
 use std::{cell::RefCell, rc::Rc};
 #[cfg(feature = "with-request-animation-frame")]
 use wasm_bindgen::closure::Closure;
@@ -51,16 +52,19 @@ where
 
     /// executed after the program has been mounted
     fn after_mounted(&self) {
-        // inject the style first
-        for style in self.app.borrow().style() {
-            Self::inject_style(&style);
-        }
-
         // call the init of the component
         let cmds = self.app.borrow_mut().init();
         // then emit the cmds, so it starts executing initial calls such (ie: fetching data,
         // listening to events (resize, hashchange)
         cmds.emit(self);
+
+        // inject the style style after call the init of the app as
+        // it may be modifying the app state including the style
+        let style = self.app.borrow().style();
+        if !style.trim().is_empty() {
+            let type_id = TypeId::of::<APP>();
+            Self::inject_style(type_id, &style);
+        }
     }
 
     /// get the real DOM node where this app is mounted to.
@@ -113,7 +117,7 @@ where
         // update the app and emit the cmd returned from the update
         let cmd = self.app.borrow_mut().update(msg);
 
-        if cmd.should_update_view {
+        if cmd.modifier.should_update_view {
             //trace!("Executing cmd..");
             let t2 = crate::now();
 
@@ -129,7 +133,8 @@ where
             log::trace!("creating app view took: {}ms", t3 - t2);
 
             // update the last DOM node tree with this new view
-            self.dom_updater.borrow_mut().update_dom(self, view);
+            let total_patches =
+                self.dom_updater.borrow_mut().update_dom(self, view);
             let t4 = crate::now();
             #[cfg(feature = "with-measure")]
             log::trace!("dom update took: {}ms", t4 - t3);
@@ -145,18 +150,19 @@ where
                 }
             }
 
-            if cmd.log_measurements {
+            if cmd.modifier.log_measurements {
                 let measurements = Measurements {
+                    name: cmd.modifier.measurement_name.clone(),
                     view_node_count: node_count,
                     update_dispatch_took: t2 - t1,
                     build_view_took: t3 - t2,
+                    total_patches,
                     dom_update_took: t4 - t3,
                     total_time: t4 - t1,
                 };
                 // tell the app on app performance measurements
-                let mut cmd_measurement =
-                    self.app.borrow().measurements(measurements);
-                cmd_measurement.should_update_view = false;
+                let cmd_measurement =
+                    self.app.borrow().measurements(measurements).no_render();
                 cmd_measurement.emit(self);
             } else {
                 #[cfg(any(feature = "with-debug", feature = "with-measure"))]
@@ -169,15 +175,17 @@ where
         cmd.emit(self);
     }
 
-    fn inject_style(style: &str) {
+    fn inject_style(type_id: TypeId, style: &str) {
         use wasm_bindgen::JsCast;
+        dbg!(&type_id);
+        let type_id = format!("{:?}", type_id);
 
         let document = crate::document();
         let html_style = document
             .create_element("style")
             .expect("must be able to create style element");
         html_style
-            .set_attribute("name", "style_name")
+            .set_attribute("class", &type_id)
             .expect("must set attribute");
         let html_style: web_sys::Node = html_style.unchecked_into();
         html_style.set_text_content(Some(style));
